#!/usr/bin/perl -w

use strict;

#BEGIN {
#  use Cwd;
#  my $pwd = getcwd();
#  use lib "$pwd/handlers";
#  
#  opendir my $dir, "$pwd/handlers";
#  foreach my $handler ( readdir($dir) )
#  {
#    print "Loading handler '$handler'...\n";
#    $handler =~ s/\.pm$//;
#    eval "use $handler";
#    die "Cannot load $handler: $@" if $@;
#  }# end foreach()
#}

1;# return true:
